unit VDD;

interface

uses
  Windows, SysUtils;

// NT_VDD.H

{ IO port service prototypes and data structure definitions }

{ Basic typedefs of VDD IO hooks }
type
  PFNVDD_INB = ^FNVDD_INB;
  FNVDD_INB = procedure(iport: Word; var data: Byte);
  PFNVDD_INW = ^FNVDD_INW;
  FNVDD_INW = procedure(iport: Word; var data: Word);
  PFNVDD_INSB = ^FNVDD_INSB;
  FNVDD_INSB = procedure(iport: Word; data: PByteArray; count: Word);
  PFNVDD_INSW = ^FNVDD_INSW;
  FNVDD_INSW = procedure(iport: Word; data: PWordArray; count: Word);
  PFNVDD_OUTB = ^FNVDD_OUTB;
  FNVDD_OUTB = procedure(iport: Word; data: Byte);
  PFNVDD_OUTW = ^FNVDD_OUTW;
  FNVDD_OUTW = procedure(iport: Word; data: Word);
  PFNVDD_OUTSB = ^FNVDD_OUTSB;
  FNVDD_OUTSB = procedure(iport: Word; data: PByteArray; count: Word);
  PFNVDD_OUTSW = ^FNVDD_OUTSW;
  FNVDD_OUTSW = procedure(iport: Word; data: PByteArray; count: Word);

  {  Array of handlers for VDD IO hooks. }
type
  _VDD_IO_HANDLERS = record
    inb_handler: FNVDD_INB;
    inw_handler: FNVDD_INW;
    insb_handler: FNVDD_INSB;
    insw_handler: FNVDD_INSW;
    outb_handler: FNVDD_OUTB;
    outw_handler: FNVDD_OUTW;
    outsb_handler: FNVDD_OUTSB;
    outsw_handler: FNVDD_OUTSW;
  end;
  VDD_IO_HANDLERS = _VDD_IO_HANDLERS;
  PVDD_IO_HANDLERS = ^VDD_IO_HANDLERS;

  { Port Range structure }
type
  _VDD_IO_PORTRANGE = record
    First: Word;
    Last: Word;
  end;
  VDD_IO_PORTRANGE = _VDD_IO_PORTRANGE;
  PVDD_IO_PORTRANGE = ^VDD_IO_PORTRANGE;

function VDDInstallIOHook(hVdd: THandle; cPortRange: Word; var pPortRange:
  VDD_IO_PORTRANGE; var IOHandler: VDD_IO_HANDLERS): BOOL; stdcall;
procedure VDDDeInstallIOHook(hVdd: THandle; cPortRange: Word; var pPortRange:
  VDD_IO_PORTRANGE); stdcall;
function VDDReserveIrqLine(hVdd: THandle; IrqLine: Word): Word; stdcall;
function VDDReleaseIrqLine(hVd: THandle; IrqLine: Word): BOOL; stdcall;

{ DMA service prototypes and data structure definitions }

{ Buffer definition for returning DMA information }
type
  _VDD_DMA_INFO = record
    addr: Word;
    count: Word;
    page: Word;
    status: Byte;
    mode: Byte;
    mask: Byte;
  end;
  VDD_DMA_INFO = _VDD_DMA_INFO;
  PVDD_DMA_INFO = ^VDD_DMA_INFO;

  { bits for querying the DMA information }
const
  VDD_DMA_ADDR = $01;
  VDD_DMA_COUNT = $02;
  VDD_DMA_PAGE = $04;
  VDD_DMA_STATUS = $08;
  VDD_DMA_ALL = VDD_DMA_ADDR or
    VDD_DMA_COUNT or
    VDD_DMA_PAGE or
    VDD_DMA_STATUS;

function VDDRequestDMA(hVdd: THandle; iChannel: Word; Buffer: Pointer; Length:
  DWORD): DWORD; stdcall;
function VDDSetDMA(hVdd: THandle; iChannel: Word; fDMA: Word; var Buffer:
  VDD_DMA_INFO): BOOL; stdcall;
function VDDQueryDMA(hVdd: THandle; iChannel: Word; var pDmaInfo: VDD_DMA_INFO):
  BOOL; stdcall;

{ Memory mapped I/O service prototypes and data structure definitions }

type
  PVDD_MEMORY_HANDLER = ^VDD_MEMORY_HANDLER;
  VDD_MEMORY_HANDLER = procedure(FaultAddress: ULONG; RWMode: ULONG);

function VDDDeInstallMemoryHook(hVdd: THandle; pStart: Pointer; Count: DWORD):
  BOOL; stdcall;
function VDDInstallMemoryHook(hVdd: THandle; pStart: Pointer; count: DWORD; var
  MemoryHandler: VDD_MEMORY_HANDLER): BOOL; stdcall;
function VDDAllocMem(hVdd: THandle; Address: Pointer; Size: DWORD): BOOL;
stdcall;
function VDDFreeMem(hVdd: THandle; Address: Pointer; Size: DWORD): BOOL;
stdcall;

{ Misc. service prototypes and data structure definitions }

function VDDIncludeMem(hVdd: THandle; Address: Pointer; Size: DWORD): BOOL;
stdcall;
function VDDExcludeMem(hVdd: THandle; Address: Pointer; Size: DWORD): BOOL;
stdcall;
procedure VDDTerminateVDM; stdcall;

{ Basic typedefs of VDD User hooks }
type
  PFNVDD_UCREATE = ^FNVDD_UCREATE;
  FNVDD_UCREATE = procedure(DosPDB: Word);
  PFNVDD_UTERMINATE = ^FNVDD_UTERMINATE;
  FNVDD_UTERMINATE = procedure(DosPDB: Word);
  PFNVDD_UBLOCK = ^FNVDD_UBLOCK;
  FNVDD_UBLOCK = procedure;
  PFNVDD_URESUME = ^FNVDD_URESUME;
  FNVDD_URESUME = procedure;

  {  Array of handlers for VDD User hooks. }

type
  PVDD_USER_HANDLERS = ^VDD_USER_HANDLERS;
  _VDD_USER_HANDLERS = record
    hvdd: THandle;
    ucr_handler: PFNVDD_UCREATE;
    uterm_handler: PFNVDD_UTERMINATE;
    ublock_handler: PFNVDD_UBLOCK;
    uresume_handler: PFNVDD_URESUME;
    next: PVDD_USER_HANDLERS;
  end;
  VDD_USER_HANDLERS = _VDD_USER_HANDLERS;

  { Function prototypes }

function VDDInstallUserHook(hVdd: THandle; var Ucr_Handler: FNVDD_UCREATE; var
  Uterm_Handler: FNVDD_UTERMINATE; var Ublock_Handler: FNVDD_UBLOCK; var
  Uresume_Handler: FNVDD_URESUME): BOOL; stdcall;
function VDDDeInstallUserHook(hVdd: Thandle): BOOL; stdcall;

procedure VDDTerminateUserHook(DosPDB: Word); stdcall;
procedure VDDCreateUserHook(DosPDB: Word); stdcall;
procedure VDDBlockUserHook; stdcall;
procedure VDDResumeUserHook; stdcall;

procedure VDDSimulate16; stdcall;

function VDDAllocateDosHandle(pPDB: ULONG; var ppSFT: Pointer; var ppJFT:
  Pointer): SHORT; stdcall;
procedure VDDAssociateNtHandle(pSFT: Pointer; h32File: THandle; wAccess: Word);
stdcall;
function VDDReleaseDosHandle(pPDB: ULONG; hFile: SHORT): BOOL; stdcall;
function VDDRetrieveNtHandle(pPDB: ULONG; hFile: SHORT; var ppSFT: Pointer; var
  ppJFT: Pointer): THandle; stdcall;

procedure VdmTraceEvent(AType: Word; wData: Word; lData: ULONG); stdcall;

procedure VDM_TRACE(AType: Word; wData: Word; lData: ULONG);

type
  _VDM_MODE = (VDM_V86, VDM_PM);
  VDM_MODE = _VDM_MODE;

type
  _VDM_ERROR_TYPE = (VDM_NO_ERROR,
    VDM_ERROR_INVALID_BUFFER_SIZE,
    VDM_ERROR_INVALID_FUNCTION);
  VDM_ERROR_TYPE = _VDM_ERROR_TYPE;

type
  _VDM_INFO_TYPE = (VDM_GET_TICK_COUNT,
    VDM_GET_TIMER0_INITIAL_COUNT,
    VDM_GET_LAST_UPDATED_TIMER0_COUNT,
    VDM_LATCH_TIMER0_COUNT,
    VDM_SET_NEXT_TIMER0_COUNT);
  VDM_INFO_TYPE = _VDM_INFO_TYPE;

const
  MSW_PE = 1;

function GetMode: VDM_MODE;

function VdmMapFlat(selector: Word; offset: ULONG; mode: VDM_MODE): Pointer;
stdcall;
function VdmUnmapFlat(selector: Word; offset: ULONG; buffer: Pointer; mode:
  VDM_MODE): BOOL; stdcall;
function VdmFlushCache(selector: Word; offset: ULONG; length: ULONG; mode:
  VDM_MODE): BOOL; stdcall;

function VdmParametersInfo(infotype: VDM_INFO_TYPE; var pBuffer; cBufferSize:
  ULONG): BOOL; stdcall;
function VdmGetParametersInfoError: VDM_INFO_TYPE;

// VDDSVC.H

{ Memory Accessing services }

function GetVDMAddress(usSeg, usOff: ULONG): Pointer; stdcall;
function GetVDMPointer(Address: ULONG; Size: ULONG; Mode: BOOL): Pointer;
stdcall;
function FlushVDMPointer(Address: ULONG; Size: Word; Buffer: Pointer; Mode:
  BOOL): BOOL; stdcall;
function FreeVDMPointer(Address: ULONG; Size: Word; Buffer: Pointer; Mode:
  BOOL): BOOL; stdcall;

{ interrupt simualtion services }

const
  ICA_MASTER = 0;
  ICA_SLAVE = 1;

procedure VDDSimulateInterrupt(ms: Integer; line: Byte; count: Integer);
stdcall;

{ Real function prototype declarations }

{ interrupt simulation functions }

procedure call_ica_hw_interrupt(ms: Integer; line: Byte; count: Integer);
stdcall;

function Sim32FreeVDMPointer(Address: ULONG; Size: Word; Buffer: Pointer; Mode:
  Boolean): BOOL; stdcall;

{ memory address manipulation functions }

function Sim32GetVDMPointer(Address: ULONG; Size: ULONG; ProtectedMode:
  Boolean): Pointer; stdcall;
function Sim32FlushVDMPointer(Address: ULONG; Size: Word; Buffer: Pointer;
  ProtectedMode: Boolean): Boolean; stdcall;

{$IFDEF CPU386}
function MGetVDMPointer(Address: ULONG; Size: ULONG; ProtectedMode: Boolean):
  Pointer; stdcall;
{$ENDIF}

{ Register manipulation functions }

function getAF: ULONG; stdcall;
function getAH: UCHAR; stdcall;
function getAL: UCHAR; stdcall;
function getAX: Word; stdcall;
function getBH: UCHAR; stdcall;
function getBL: UCHAR; stdcall;
function getBP: Word; stdcall;
function getBX: Word; stdcall;
function getCF: ULONG; stdcall;
function getCH: UCHAR; stdcall;
function getCL: UCHAR; stdcall;
function getCS: Word; stdcall;
function getCX: Word; stdcall;
function getDF: ULONG; stdcall;
function getDH: UCHAR; stdcall;
function getDI: Word; stdcall;
function getDL: UCHAR; stdcall;
function getDS: Word; stdcall;
function getDX: Word; stdcall;
function getEAX: ULONG; stdcall;
function getEBP: ULONG; stdcall;
function getEBX: ULONG; stdcall;
function getECX: ULONG; stdcall;
function getEDI: ULONG; stdcall;
function getEDX: ULONG; stdcall;
function getEFLAGS: ULONG; stdcall;
function getEIP: ULONG; stdcall;
function getES: Word; stdcall;
function getESI: ULONG; stdcall;
function getESP: ULONG; stdcall;
function getFS: Word; stdcall;
function getGS: Word; stdcall;
function getIF: ULONG; stdcall;
function getIP: Word; stdcall;
function getMSW: UCHAR; stdcall;
function getOF: ULONG; stdcall;
function getPF: ULONG; stdcall;
function getSF: ULONG; stdcall;
function getSI: Word; stdcall;
function getSP: Word; stdcall;
function getSS: Word; stdcall;
function getZF: ULONG; stdcall;
procedure setAF(Value: ULONG); stdcall;
procedure setAH(Value: UCHAR); stdcall;
procedure setAL(Value: UCHAR); stdcall;
procedure setAX(Value: Word); stdcall;
procedure setBH(Value: UCHAR); stdcall;
procedure setBL(Value: UCHAR); stdcall;
procedure setBP(Value: Word); stdcall;
procedure setBX(Value: Word); stdcall;
procedure setCF(Value: ULONG); stdcall;
procedure setCH(Value: UCHAR); stdcall;
procedure setCL(Value: UCHAR); stdcall;
procedure setCS(Value: Word); stdcall;
procedure setCX(Value: Word); stdcall;
procedure setDF(Value: ULONG); stdcall;
procedure setDH(Value: UCHAR); stdcall;
procedure setDI(Value: Word); stdcall;
procedure setDL(Value: UCHAR); stdcall;
procedure setDS(Value: Word); stdcall;
procedure setDX(Value: Word); stdcall;
procedure setEAX(Value: ULONG); stdcall;
procedure setEBP(Value: ULONG); stdcall;
procedure setEBX(Value: ULONG); stdcall;
procedure setECX(Value: ULONG); stdcall;
procedure setEDI(Value: ULONG); stdcall;
procedure setEDX(Value: ULONG); stdcall;
procedure setEFLAGS(Value: ULONG); stdcall;
procedure setEIP(Value: ULONG); stdcall;
procedure setES(Value: Word); stdcall;
procedure setESI(Value: ULONG); stdcall;
procedure setESP(Value: ULONG); stdcall;
procedure setFS(Value: Word); stdcall;
procedure setGS(Value: Word); stdcall;
procedure setIF(Value: ULONG); stdcall;
procedure setIP(Value: Word); stdcall;
procedure setMSW(Value: UCHAR); stdcall;
procedure setOF(Value: ULONG); stdcall;
procedure setPF(Value: ULONG); stdcall;
procedure setSF(Value: ULONG); stdcall;
procedure setSI(Value: Word); stdcall;
procedure setSP(Value: Word); stdcall;
procedure setSS(Value: Word); stdcall;
procedure setZF(Value: ULONG); stdcall;

function VDDAvailable: Boolean;

type
  TVDDMapMem = function(selector: Word; offset: ULONG; mode: VDM_MODE): Pointer;
  stdcall;
  TVDDUnmapMem = function(selector: Word; offset: ULONG; buffer: Pointer; mode:
    VDM_MODE): BOOL; stdcall;
  TVDDFlushMem = function(selector: Word; offset: ULONG; length: ULONG; mode:
    VDM_MODE): BOOL; stdcall;

var
  VDDMapMem: TVDDMapMem;
  VDDUnmapMem: TVDDUnmapMem;
  VDDFlushMem: TVDDFlushMem;

function VDDInitialization: Boolean;
procedure VDDFinalization;

function GetVDDHWnd: HWnd;


type
  TVDDCallHook = procedure(AFunction: Word); stdcall;

var
  VDDCallHook: TVDDCallHook = nil;

implementation

const
  ntvdm = 'NTVDM.EXE';

var
  VerInfo: TOSVersionInfo;
  NT4Present: Boolean;

var
  VDDHWnd: HWnd;

var
  VDDWndClass: TWndClass = (
    Style: 0;
    lpfnWndProc: @DefWindowProc;
    cbClsExtra: 0;
    cbWndExtra: 0;
    hInstance: 0;
    hIcon: 0;
    hCursor: 0;
    hbrBackground: 0;
    lpszMenuName: nil;
    lpszClassName: 'VDDWindow');

procedure RequestVDDHWnd;
var
  Wnd: TWndClass;
  AlreadyDefined: Boolean;
begin
  VDDWndClass.hInstance := HInstance;
  AlreadyDefined := GetClassInfo(HInstance, VDDWndClass.lpszClassName, Wnd);

  if not AlreadyDefined or (Wnd.lpfnWndProc <> @DefWindowProc) then
  begin
    if AlreadyDefined then
      UnregisterClass(VDDWndClass.lpszClassName, HInstance);
    RegisterClass(VDDWndClass);
  end;

  VDDHWnd := CreateWindowEx(WS_EX_TOOLWINDOW, VDDWndClass.lpszClassName, '',
    WS_POPUP, 0, 0, 0, 0, 0, 0, HInstance, nil);
end;

function GetVDDHWnd;
begin
  Result := VDDHWnd;
end;

function VDDInitialization;
begin
  if VDDAvailable then
  begin
    RequestVDDHWnd;
    Result := (VDDHWnd <> 0);
  end
  else
    Result := False;
end;

procedure VDDFinalization;
begin
  if VDDHWnd <> 0 then
    DestroyWindow(VDDHWnd);
end;


procedure VDM_TRACE(AType: Word; wData: Word; lData: ULONG);
begin
{$IFDEF DBG}
  VdmTraceEvent(AType, wData, lData);
{$ENDIF}
end;

function GetMode: VDM_MODE;
begin
  if getMSW and MSW_PE <> 0 then
    Result := VDM_PM
  else
    Result := VDM_V86;
end;

{$IFDEF CPU386}
function VdmUnmapFlat;
begin
  Result := True;
end;

function VdmFlushCache;
begin
  Result := True;
end;
{$ELSE}
function VdmUnmapFlat; external ntvdm name 'VdmUnmapFlat';
function VdmFlushCache; external ntvdm name 'VdmFlushCache';
{$ENDIF}

function GetVDMAddress;
begin
  Result := Ptr(usSeg shl 4 + usOff);
end;

function GetVDMPointer;
begin
  Result := Sim32GetVDMPointer(Address, Size, Boolean(Mode));
end;

function FlushVDMPointer;
begin
  Result := Sim32FlushVDMPointer(Address, Size, Buffer, Mode);
end;

function FreeVDMPointer;
begin
  Result := Sim32FreeVDMPointer(Address, Size, Buffer, Mode);
end;

procedure VDDSimulateInterrupt(ms: Integer; line: Byte; count: Integer);
begin
  call_ica_hw_interrupt(ms, line, 1);
end;


function Sim32FreeVDMPointer;
begin
  Result := True;
end;

{$IFDEF CPU386}
function Sim32GetVDMPointer;
begin
  Result := MGetVdmPointer(Address, Size, ProtectedMode);
end;

function Sim32FlushVDMPointer;
begin
  Result := True;
end;
{$ELSE}
function Sim32GetVDMPointer; external ntvdm name 'Sim32GetVDMPointer';
function Sim32FlushVDMPointer; external ntvdm name 'Sim32FlushVDMPointer';
{$ENDIF}

{$IFDEF CPU386}
function MGetVdmPointer; external ntvdm name 'MGetVdmPointer';
{$ENDIF}

function VDDInstallIOHook; external ntvdm name 'VDDInstallIOHook';
procedure VDDDeInstallIOHook; external ntvdm name 'VDDDeInstallIOHook';
function VDDReserveIrqLine; external ntvdm name 'VDDReserveIrqLine';
function VDDReleaseIrqLine; external ntvdm name 'VDDReleaseIrqLine';
function VDDRequestDMA; external ntvdm name 'VDDRequestDMA';
function VDDSetDMA; external ntvdm name 'VDDSetDMA';
function VDDQueryDMA; external ntvdm name 'VDDQueryDMA';
function VDDDeInstallMemoryHook; external ntvdm name 'VDDDeInstallMemoryHook';
function VDDInstallMemoryHook; external ntvdm name 'VDDInstallMemoryHook';
function VDDAllocMem; external ntvdm name 'VDDAllocMem';
function VDDFreeMem; external ntvdm name 'VDDFreeMem';
function VDDIncludeMem; external ntvdm name 'VDDIncludeMem';
function VDDExcludeMem; external ntvdm name 'VDDExcludeMem';
procedure VDDTerminateVDM; external ntvdm name 'VDDTerminateVDM';
function VDDInstallUserHook; external ntvdm name 'VDDInstallUserHook';
function VDDDeInstallUserHook; external ntvdm name 'VDDDeinstallUserHook';
procedure VDDTerminateUserHook; external ntvdm name 'VDDTerminateUserHook';
procedure VDDCreateUserHook; external ntvdm name 'VDDCreateUserHook';
procedure VDDBlockUserHook; external ntvdm name 'VDDBlockUserHook';
procedure VDDResumeUserHook; external ntvdm name 'VDDResumeUserHook';
procedure VDDSimulate16; external ntvdm name 'VDDSimulate16';
function VDDAllocateDosHandle; external ntvdm name 'VDDAllocateDosHandle';
procedure VDDAssociateNtHandle; external ntvdm name 'VDDAssociateNtHandle';
function VDDReleaseDosHandle; external ntvdm name 'VDDReleaseDosHandle';
function VDDRetrieveNtHandle; external ntvdm name 'VDDRetrieveNtHandle';
procedure VdmTraceEvent; external ntvdm name 'VdmTraceEvent';
function VdmMapFlat; external ntvdm name 'VdmMapFlat';
function VdmParametersInfo; external ntvdm name 'VdmParametersInfo';
function VdmGetParametersInfoError; external ntvdm name
  'VdmGetParametersInfoError';
procedure call_ica_hw_interrupt; external ntvdm name 'call_ica_hw_interrupt';

function getAF; external ntvdm name 'getAF';
function getAH; external ntvdm name 'getAH';
function getAL; external ntvdm name 'getAL';
function getAX; external ntvdm name 'getAX';
function getBH; external ntvdm name 'getBH';
function getBL; external ntvdm name 'getBL';
function getBP; external ntvdm name 'getBP';
function getBX; external ntvdm name 'getBX';
function getCF; external ntvdm name 'getCF';
function getCH; external ntvdm name 'getCH';
function getCL; external ntvdm name 'getCL';
function getCS; external ntvdm name 'getCS';
function getCX; external ntvdm name 'getCX';
function getDF; external ntvdm name 'getDF';
function getDH; external ntvdm name 'getDH';
function getDI; external ntvdm name 'getDI';
function getDL; external ntvdm name 'getDL';
function getDS; external ntvdm name 'getDS';
function getDX; external ntvdm name 'getDX';
function getEAX; external ntvdm name 'getEAX';
function getEBP; external ntvdm name 'getEBP';
function getEBX; external ntvdm name 'getEBX';
function getECX; external ntvdm name 'getECX';
function getEDI; external ntvdm name 'getEDI';
function getEDX; external ntvdm name 'getEDX';
function getEFLAGS; external ntvdm name 'getEFLAGS';
function getEIP; external ntvdm name 'getEIP';
function getES; external ntvdm name 'getES';
function getESI; external ntvdm name 'getESI';
function getESP; external ntvdm name 'getESP';
function getFS; external ntvdm name 'getFS';
function getGS; external ntvdm name 'getGS';
function getIF; external ntvdm name 'getIF';
function getIP; external ntvdm name 'getIP';
function getMSW; external ntvdm name 'getMSW';
function getOF; external ntvdm name 'getOF';
function getPF; external ntvdm name 'getPF';
function getSF; external ntvdm name 'getSF';
function getSI; external ntvdm name 'getSI';
function getSP; external ntvdm name 'getSP';
function getSS; external ntvdm name 'getSS';
function getZF; external ntvdm name 'getZF';
procedure setAF; external ntvdm name 'setAF';
procedure setAH; external ntvdm name 'setAH';
procedure setAL; external ntvdm name 'setAL';
procedure setAX; external ntvdm name 'setAX';
procedure setBH; external ntvdm name 'setBH';
procedure setBL; external ntvdm name 'setBL';
procedure setBP; external ntvdm name 'setBP';
procedure setBX; external ntvdm name 'setBX';
procedure setCF; external ntvdm name 'setCF';
procedure setCH; external ntvdm name 'setCH';
procedure setCL; external ntvdm name 'setCL';
procedure setCS; external ntvdm name 'setCS';
procedure setCX; external ntvdm name 'setCX';
procedure setDF; external ntvdm name 'setDF';
procedure setDH; external ntvdm name 'setDH';
procedure setDI; external ntvdm name 'setDI';
procedure setDL; external ntvdm name 'setDL';
procedure setDS; external ntvdm name 'setDS';
procedure setDX; external ntvdm name 'setDX';
procedure setEAX; external ntvdm name 'setEAX';
procedure setEBP; external ntvdm name 'setEBP';
procedure setEBX; external ntvdm name 'setEBX';
procedure setECX; external ntvdm name 'setECX';
procedure setEDI; external ntvdm name 'setEDI';
procedure setEDX; external ntvdm name 'setEDX';
procedure setEFLAGS; external ntvdm name 'setEFLAGS';
procedure setEIP; external ntvdm name 'setEIP';
procedure setES; external ntvdm name 'setES';
procedure setESI; external ntvdm name 'setESI';
procedure setESP; external ntvdm name 'setESP';
procedure setFS; external ntvdm name 'setFS';
procedure setGS; external ntvdm name 'setGS';
procedure setIF; external ntvdm name 'setIF';
procedure setIP; external ntvdm name 'setIP';
procedure setMSW; external ntvdm name 'setMSW';
procedure setOF; external ntvdm name 'setOF';
procedure setPF; external ntvdm name 'setPF';
procedure setSF; external ntvdm name 'setSF';
procedure setSI; external ntvdm name 'setSI';
procedure setSP; external ntvdm name 'setSP';
procedure setSS; external ntvdm name 'setSS';
procedure setZF; external ntvdm name 'setZF';

function VDDMapMemNT4(selector: Word; offset: ULONG; mode: VDM_MODE): Pointer;
  stdcall;
begin
  Result := GetVDMPointer(Integer(GetVDMAddress(selector, offset)), 65535,
    BOOL(mode));
end;

function VDDUnmapMemNT4(selector: Word; offset: ULONG; buffer: Pointer; mode:
  VDM_MODE): BOOL; stdcall;
begin
  Result := FreeVDMPointer(Integer(GetVDMAddress(selector, offset)), 65535,
    buffer, BOOL(mode));
end;

function VDDFlushMemNT4(selector: Word; offset: ULONG; length: ULONG; mode:
  VDM_MODE): BOOL; stdcall;
begin
{$IFDEF CPU386}
  Result := True;
{$ELSE}
  raise
    EOSError.Create('Fonction VDDFlushMem non supporte sous environnement Windows NT4');
{$ENDIF}
end;

function NullVDDMapMem(selector: Word; offset: ULONG; mode: VDM_MODE): Pointer;
  stdcall;
begin
  Result := nil;
end;

function NullVDDUnmapMem(selector: Word; offset: ULONG; buffer: Pointer; mode:
  VDM_MODE): BOOL; stdcall;
begin
  Result := False;
end;

function NullVDDFlushMem(selector: Word; offset: ULONG; length: ULONG; mode:
  VDM_MODE): BOOL; stdcall;
begin
  Result := False;
end;

procedure SetOSSpecificFunctions;
begin
  if not NT4Present then
  begin
    VDDMapMem := NullVDDMapMem;
    VDDUnmapMem := NullVDDUnmapMem;
    VDDFlushMem := NullVDDFlushMem;
  end
  else
  begin
    if (VerInfo.dwMajorVersion = 4) and
      (VerInfo.dwMinorVersion = 0) then
    begin
      VDDMapMem := VDDMapMemNT4;
      VDDUnmapMem := VDDUnmapMemNT4;
      VDDFlushMem := VDDFlushMemNT4;
    end
    else
    begin
      VDDMapMem := VdmMapFlat;
      VDDUnmapMem := VdmUnmapFlat;
      VDDFlushMem := VdmFlushCache;
    end;
  end;
end;

function VDDAvailable: Boolean;
begin
  Result := NT4Present;
end;

initialization
  VerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  if GetVersionEx(VerInfo) then
    NT4Present := (VerInfo.dwPlatformId = VER_PLATFORM_WIN32_NT) and
      (VerInfo.dwMajorVersion >= 4)
  else
    NT4Present := False;

  SetOSSpecificFunctions;
end.


