{************************************************************}
{                                                            }
{ Unit VDDClipbrd                                            }
{                                                            }
{ Unit de gestion du presse-papiers pour l'interface        }
{ Dos - Windows grce  un VDD                               }
{ Reprend l'interface de l'int 2Fh, fonctions 1700h  170Ah  }
{ avec support supplmentaire des transferts de plus de 64Ko }
{ (nouvelles fonctions 1710h et 1711h)                       }
{                                                            }
{ Copyright Eric Sigoillot, 2004 - Tous droits rservs      }
{                                                            }
{************************************************************}
unit VDDClipbrd;

interface

uses
  Windows;

function VDDClpInitialization: Boolean;
procedure VDDClpFinalization;

{ ClpGetVersion: Donne la version de WinOldAp
    Entre: aucune (1700h)
    Sortie: AX }
procedure ClpGetVersion; stdcall;

{ ClpOpenClipbrd: Ouvre le presse-papiers
    Entre: aucune (1701h)
    Sortie: AX }
procedure ClpOpenClipbrd; stdcall;

{ ClpEmptyClipbrd: Vide le presse-papiers
    Entre: aucune (1702h)
    Sortie: AX }
procedure ClpEmptyClipbrd; stdcall;

{ ClpSetClipbrdData: Place des donnes dans le presse-papiers
    Entre: BX, CX, DX, SI, ES (1703h)
    Sortie: AX }
procedure ClpSetClipbrdData; stdcall;

{ ClpGetClipbrdDataSize: Donne la taille des donnnes dans le prese-papiers
    Entre: DX (1704h)
    Sortie: AX, DX }
procedure ClpGetClipbrdDataSize; stdcall;

{ ClpGetClipbrdData: Stocke les donnes contenues dans le presse-papiers
    Entre: BX, DX, ES (1705h)
    Sortie: AX, DX }
procedure ClpGetClipbrdData; stdcall;

{ ClpCloseClipbrd: Ferme le presse-papiers
    Entre: aucune (1708h)
    Sortie: AX }
procedure ClpCloseClipbrd; stdcall;

{ ClpCompactClipbrd: Essaie de compresser le contenu du presse-papiers
    Entre: CX, SI (1709h)
    Sortie: AX, DX }
procedure ClpCompactClipbrd; stdcall;

{ ClpGetDeviceCaps: Renvoie des informations sur le pilote video Windows
    Entre: DX (170Ah)
    Sortie: AX }
procedure ClpGetDeviceCaps; stdcall;

{ ClpGetLargeData: Permet de rcuprer des donnes de plus de 64 Ko
    Entre: BX, CX, DX, ES (1710h)
    Sortie: AX }
procedure ClpGetLargeData; stdcall;

{ ClpSetLargeData: Permet de stocker des donnes de plus de 64 Ko
    Entre: BX, CX, DX, SI, ES (1711h)
    Sortie: AX }
procedure ClpSetLargeData; stdcall;


implementation

uses
  VDD;

var
  OldVDDCallHook: TVDDCallHook;

var
  hData: HGlobal;
  hBlock: Integer;
  hBlockCount: Integer;
  hBlockSize: Word;

procedure VDDClpCallHook(AFunction: Word); stdcall;
begin
  case AFunction of
    $1700: ClpGetVersion;
    $1701: ClpOpenClipbrd;
    $1702: ClpEmptyClipbrd;
    $1703: ClpSetClipbrdData;
    $1704: ClpGetClipbrdDataSize;
    $1705: ClpGetClipbrdData;
    $1708: ClpCloseClipbrd;
    $1709: ClpCompactClipbrd;
    $170A: ClpGetDeviceCaps;
    $1710: ClpSetLargeData;
    $1711: ClpGetLargeData;
  else
    begin
      if Assigned(OldVDDCallHook) then
        OldVDDCallHook(AFunction);
    end;
  end;
end;

function VDDClpInitialization: Boolean;
begin
  hData := 0;
  OldVDDCallHook := VDDCallHook;
  VDDCallHook := VDDClpCallHook;

  Result := True;
end;

procedure VDDClpFinalization;
begin
  VDDCallHook := OldVDDCallHook;
end;

procedure ClpGetVersion;
begin
  { Indique un WinOldApp de version 3.1 }
  SetAL(3);
  SetAH(1);
end;

procedure ClpOpenClipbrd;
begin
  SetAX(Word(OpenClipboard(GetVDDHwnd)));
end;

procedure ClpEmptyClipbrd;
begin
  SetAX(Word(EmptyClipboard));
end;

procedure ClpSetClipbrdData;
var
  hMem: HGlobal;
  hPtr, VDMPtr: Pointer;
  Size: Cardinal;
begin
  Size := Cardinal(GetSI) shl 16 + Cardinal(GetCX);
  VDMPtr := VDDMapMem(GetES, GetBX, VDM_V86);

  hMem := GlobalAlloc(GMEM_MOVEABLE or GMEM_DDESHARE, Size);
  hPtr := GlobalLock(hMem);
  if (VDMPtr <> nil) and (hPtr <> nil) then
  begin
    Move(VDMPtr^, hPtr^, Size);
    GlobalUnlock(hMem);
    SetAX(Word(SetClipboardData(GetDX, hMem)));
    VDDUnmapMem(GetES, GetBX, VDMPtr, VDM_V86);
  end
  else
    SetAX(0);
end;

procedure ClpGetClipbrdDataSize;
var
  hMem: HGlobal;
  Size: Integer;
begin
  hMem := GetClipboardData(GetDX);
  if hMem <> 0 then
  begin
    Size := GlobalSize(hMem);
    SetDX((Size shr 16) and $FFFF);
    SetAX(Size and $FFFF);
  end
  else
  begin
    SetAX(0);
    SetDX(0);
  end;
end;

procedure ClpGetClipbrdData;
var
  hMem: HGlobal;
  Size: Integer;
  hPtr, VDMPtr: Pointer;
begin
  SetAX(0);
  hMem := GetClipboardData(GetDX);
  if hMem <> 0 then
  begin
    Size := GlobalSize(hMem);
    if Size > 65520 then
      Size := 65520;
    hPtr := GlobalLock(hMem);
    if hPtr <> nil then
    begin
      VDMPtr := VDDMapMem(GetES, GetBX, VDM_V86);
      if VDMPtr <> nil then
      begin
        Move(hPtr^, VDMPtr^, Size);
        GlobalUnlock(hMem);
        SetAX(hMem);
        VDDUnmapMem(GetES, GetBX, VDMPtr, VDM_V86);
      end;
    end;
  end;
end;

procedure ClpCloseClipbrd;
begin
  SetAX(Word(CloseClipboard));
end;

procedure ClpCompactClipbrd;
begin

end;

procedure ClpGetDeviceCaps;
begin
  SetAX(GetDeviceCaps(GetWindowDC(GetVDDHWnd), GetDX));
end;

procedure ClpInitLargeGetTransfer;
var
  hMem: HGlobal;
  hSrcPtr, hDstPtr: Pointer;
  Size: Integer;
begin
  SetAX(0);
  if hData <> 0 then
    Exit;
  hMem := GetClipboardData(GetDX);
  if hMem <> 0 then
  begin
    Size := GlobalSize(hMem);
    hData := GlobalAlloc(GMEM_MOVEABLE, Size);
    if hData <> 0 then
    begin
      hSrcPtr := GlobalLock(hMem);
      hDstPtr := GlobalLock(hData);
      if (hSrcPtr <> nil) and (hDstPtr <> nil) then
        CopyMemory(hDstPtr, hSrcPtr, Size);
      GlobalUnlock(hMem);
      GlobalUnlock(hData);

      hBlock := 0;
      hBlockSize := GetCX;
      hBlockCount := Size div hBlockSize;
      if Size mod hBlockSize > 0 then
        Inc(hBlockCount);

      SetAX(hData);
      SetCX(hBlockCount and $FFFF);
      SetSI((hBlockCount shr 16) and $FFFF);
    end;
  end
end;

procedure ClpGetLargeData;
var
  hPtr, VDMPtr: Pointer;
  Size: Word;
begin
  SetAX(0);

  if (GetES = 0) and (GetBX = 0) then
  begin
    ClpInitLargeGetTransfer;
    Exit;
  end;

  if hData = 0 then
    Exit;
  hPtr := GlobalLock(hData);
  if hPtr <> nil then
  begin
    VDMPtr := VDDMapMem(GetES, GetBX, VDM_V86);
    if VDMPtr <> nil then
    begin
      if hBlock < hBlockCount - 1 then
        Size := hBlockSize
      else
        Size := GlobalSize(hData) mod hBlockSize;
      Move(Ptr(Integer(hPtr) + hBlock * hBlockSize)^, VDMPtr^, Size);
      Inc(hBlock);
      SetAX(Size);
      VDDUnmapMem(GetES, GetBX, VDMPtr, VDM_V86);
    end;
    GlobalUnlock(hData);
    if hBlock = hBlockCount then
    begin
      GlobalFree(hData);
      hData := 0;
      hBlock := 0;
      hBlockCount := 0;
    end;
  end;
end;

procedure ClpInitLargeSetTransfer;
var
  Size: Integer;
begin
  SetAX(0);
  if hData <> 0 then
    Exit;

  Size := Cardinal(GetSI) shl 16 + Cardinal(GetCX);
  hData := GlobalAlloc(GMEM_MOVEABLE or GMEM_DDESHARE, Size);
  if hData <> 0 then
  begin
    hBlockSize := GetDX;
    hBlockCount := Size div hBlockSize;
    if Size mod hBlockSize <> 0 then
      Inc(hBlockCount);
    hBlock := 0;
    SetAX(Word(hData <> 0));
  end;
end;

procedure ClpSetLargeData;
var
  hPtr, VDMPtr: Pointer;
  Size: Word;
begin
  SetAX(0);
  if (GetES = 0) and (GetBX = 0) then
  begin
    ClpInitLargeSetTransfer;
    Exit;
  end;

  if hData = 0 then
    Exit;

  hPtr := GlobalLock(hData);
  if hPtr <> nil then
  begin
    VDMPtr := VDDMapMem(GetES, GetBX, VDM_V86);
    if VDMPtr <> nil then
    begin
      if hBlock < hBlockCount - 1 then
        Size := hBlockSize
      else
        Size := GlobalSize(hData) mod hBlockSize;
      Move(VDMPtr^, Ptr(Integer(hPtr) + hBlock * hBlockSize)^, Size);
      Inc(hBlock);
      VDDUnmapMem(GetES, GetBX, VDMPtr, VDM_V86);
    end;
    GlobalUnlock(hData);
    if hBlock = hBlockCount then
    begin
      SetAX(Word(SetClipboardData(GetDX, hData)));
      hData := 0;
      hBlock := 0;
      hBlockCount := 0;
    end;
  end;
end;

end.

